from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _Bpref(measures.BaseMeasure):
    """
    Binary Preference (Bpref).
    This measure examines the relative ranks of judged relevant and non-relevant documents. Non-judged documents are not considered. 
    """
    __name__ = 'Bpref'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


Bpref = _Bpref()
BPref = Bpref
measures.register(Bpref, ['BPref'])



reigistry = {}
def register(provider):
	reigistry[provider.NAME] = provider
	return provider

from .base import MeasureProvider
from .fallback_provider import FallbackProvider
from .pytrec_eval_provider import PytrecEvalProvider
from .judged_provider import JudgedProvider
from .gdeval_provider import GdevalProvider
from .trectools_provider import TrectoolsProvider
from .msmarco_provider import MsMarcoProvider



import unittest
import itertools
import ir_measures


class TestPytrecEval(unittest.TestCase):

    def test_NumRet(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 0
0 0 D1 1
0 0 D2 1
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 2
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D3 2 0.7 run
1 0 D4 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.NumRet
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 5)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 4)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 9)

        measure = ir_measures.NumRet(rel=1)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 3)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 1)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 4)

        measure = ir_measures.NumRet(rel=2)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 1)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 2)

        measure = ir_measures.NumRet(rel=3)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0)

    def test_NumQ(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 0
0 0 D1 1
0 0 D2 1
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 2
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D3 2 0.7 run
1 0 D4 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.NumQ
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 1)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 2)

    def test_NumRel(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 0
0 0 D1 1
0 0 D2 1
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 0
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D3 2 0.7 run
1 0 D4 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.NumRel
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 3)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 2)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 5)


    def test_Success(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 0
0 0 D1 1
0 0 D2 2
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 0
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D4 2 0.7 run
1 0 D3 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.Success@2
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.5)

        measure = ir_measures.Success(rel=2)@2
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.5)

        measure = ir_measures.Success(rel=3)@2
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0)

    def test_SetP(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 0
0 0 D1 1
0 0 D2 2
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 0
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D4 2 0.7 run
1 0 D3 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.SetP(rel=1)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, .6)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.425)

        measure = ir_measures.SetP(rel=2)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, .4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.325)

        measure = ir_measures.SetP(rel=3)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0)

    def test_IPrec(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 1
0 0 D1 1
0 0 D2 2
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D5 0
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D4 2 0.7 run
1 0 D3 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.IPrec@0.25
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1.0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.625)

        measure = ir_measures.IPrec@0.5
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1.0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.625)

        measure = ir_measures.IPrec@0.75
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertEqual(result[0].value, 1.0)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0)
        self.assertEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.5)

        measure = ir_measures.IPrec(rel=2)@0.1
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, .6666666, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.4583, places=4)

        measure = ir_measures.IPrec(rel=2)@0.25
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, .6666666, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.4583, places=4)

        measure = ir_measures.IPrec(rel=2)@0.5
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, .6666666, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.4583, places=4)

        measure = ir_measures.IPrec(rel=2)@0.75
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, .6666666, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .25)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.4583, places=4)


    def test_infAP(self):
        qrels = list(ir_measures.util.parse_trec_qrels('''
0 0 D0 1
0 0 D1 -1
0 0 D2 0
0 0 D3 2
0 0 D4 0
1 0 D0 1
1 0 D3 2
1 0 D4 -1
1 0 D5 0
'''))
        run = list(ir_measures.util.parse_trec_run('''
0 0 D0 1 0.8 run
0 0 D2 2 0.7 run
0 0 D1 3 0.3 run
0 0 D3 4 0.4 run
0 0 D4 5 0.1 run
1 0 D1 1 0.8 run
1 0 D4 2 0.7 run
1 0 D3 3 0.3 run
1 0 D2 4 0.4 run
'''))
        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.AP
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, 0.8333, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, .125)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.47916666666666663, places=4)

        measure = ir_measures.infAP
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, 0.8333, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertEqual(result[1].value, 0.1875)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.510416, places=4)

        provider = ir_measures.providers.PytrecEvalProvider()
        measure = ir_measures.AP(rel=2)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, 0.33333, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertAlmostEqual(result[1].value, 0.25, places=4)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.2917, places=4)

        measure = ir_measures.infAP(rel=2)
        result = list(provider.iter_calc([measure], qrels, run))
        self.assertEqual(result[0].query_id, "0")
        self.assertAlmostEqual(result[0].value, 0.33333, places=4)
        self.assertEqual(result[1].query_id, "1")
        self.assertAlmostEqual(result[1].value, 0.375, places=4)
        self.assertAlmostEqual(provider.calc_aggregate([measure], qrels, run)[measure], 0.3542, places=4)



if __name__ == '__main__':
    unittest.main()



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _ERR(measures.BaseMeasure):
    """
    The Expected Reciprocal Rank (ERR) is a precision-focused measure.
    TODO: finish
    """
    __name__ = 'ERR'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'cutoff': measures.ParamInfo(dtype=int, required=False, desc='ranking cutoff threshold'),
    }


ERR = _ERR()
measures.register(ERR)



import contextlib
import pytrec_eval
import ir_measures
from ir_measures import providers, measures
from ir_measures.providers.base import Any, Choices, Metric, NOT_PROVIDED


class JudgedProvider(providers.MeasureProvider):
    NAME = 'judged'
    SUPPORTED_MEASURES = [
        measures._Judged(cutoff=Any())
    ]

    @contextlib.contextmanager
    def _calc_ctxt(self, measures, qrels):
        qrels = ir_measures.util.QrelsConverter(qrels).as_dict_of_dict()

        cutoffs = []
        for measure in ir_measures.util.flatten_measures(measures):
            if measure.NAME == 'Judged':
                cutoffs.append((measure['cutoff'], measure))
            else:
                raise ValueError(f'unsupported measure {measure}')

        def _iter_calc(run):
            run = ir_measures.util.RunConverter(run).as_dict_of_dict()
            sorted_run = {q: list(sorted(run[q].items(), key=lambda x: (-x[1], x[0]))) for q in run}
            for qid in run:
                qid_qrels = qrels.get(qid, {})
                for cutoff, measure in cutoffs:
                    judged_c = sum((did in qid_qrels) for did, _ in sorted_run.get(qid, [])[:cutoff])
                    value = judged_c / cutoff
                    yield Metric(query_id=qid, measure=measure, value=value)
        yield _iter_calc

providers.register(JudgedProvider())



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _IPrec(measures.BaseMeasure):
    """
    Interpolated Precision at a given recall cutoff. Used for building precision-recall graphs.
    Unlike most measures, where @ indicates an absolute cutoff threshold, here @ sets the recall
    cutoff.
    """
    __name__ = 'IPrec'
    NAME = __name__
    AT_PARAM = 'recall'
    SUPPORTED_PARAMS = {
        'recall': measures.ParamInfo(dtype=float, required=True, desc='recall threshold'),
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


IPrec = _IPrec()
measures.register(IPrec)



import pkgutil
import subprocess
import tempfile
import contextlib
import ir_measures
from ir_measures import providers, measures
from ir_measures.providers.base import Any, Choices, Metric, NOT_PROVIDED


class GdevalProvider(providers.MeasureProvider):
    NAME = 'gdeval'
    SUPPORTED_MEASURES = [
        measures._nDCG(cutoff=Any(required=True), dcg=Choices('exp-log2')),
        measures._ERR(cutoff=Any(required=True)),
    ]

    def __init__(self):
        super().__init__()
        self._perlf = None

    @contextlib.contextmanager
    def _calc_ctxt(self, measures, qrels):
        cutoffs = {}
        for measure in ir_measures.util.flatten_measures(measures):
            if measure.NAME in ('nDCG', 'ERR'):
                cutoff = measure['cutoff']
                if cutoff not in cutoffs:
                    cutoffs[cutoff] = []
                cutoffs[cutoff].append(measure)
            else:
                raise ValueError(f'unsupported measure {measure}')

        # Convert qrels to file
        with ir_measures.util.QrelsConverter(qrels).as_tmp_file() as f_qrels:
            def _iter_calc(run):
                # Convert run to file
                with ir_measures.util.RunConverter(run).as_tmp_file() as f_run:
                    for cutoff, measures in cutoffs.items():
                        yield from self._invoke_iter(f_qrels.name, f_run.name, cutoff, measures)

            yield _iter_calc

    def _invoke_iter(self, qrelsf, runf, cutoff, measures):
        """
        Runs gdeval.pl on the given run/qrels pair
        """
        # adapted from searchivarius/AccurateLuceneBM25/scripts/eval_output_gdeval.py
        nDCG_measure, ERR_measure = None, None
        for measure in measures:
            if measure.NAME == 'nDCG':
                nDCG_measure = measure
            elif measure.NAME == 'ERR':
                ERR_measure = measure

        cmd = ['perl', self._perlf.name, qrelsf, runf, str(cutoff)]
        output = subprocess.check_output(cmd)
        output = output.decode().replace('\t', ' ').split('\n')
        for i, s in enumerate(output):
            if s == '' or i == 0:
                continue
            arr = s.split(',')
            assert len(arr) == 4
            _, qid, ndcg, err = arr
            if nDCG_measure is not None:
                yield Metric(query_id=qid, measure=nDCG_measure, value=float(ndcg))
            if ERR_measure is not None:
                yield Metric(query_id=qid, measure=ERR_measure, value=float(err))

    def initialize(self):
        try:
            subprocess.check_output(['perl', '--version'])
        except CalledProcessError as ex:
            raise RuntimeError('perl not available', ex)
        perlf_contents = pkgutil.get_data('ir_measures', 'bin/gdeval.pl')
        self._perlf = tempfile.NamedTemporaryFile()
        self._perlf.write(perlf_contents)
        self._perlf.flush()


providers.register(GdevalProvider())



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _Rprec(measures.BaseMeasure):
    """
    The precision of at R, where R is the number of relevant documents for a given query. Has the cute property that
    it is also the recall at R.
    """
    __name__ = 'Rprec'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


Rprec = _Rprec()
RPrec = Rprec
measures.register(Rprec, ['RPrec'])



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _nDCG(measures.BaseMeasure):
    """
    The normalized Discounted Cumulative Gain (nDCG).
    Uses graded labels - systems that put the highest graded documents at the top of the ranking.
    It is normalized wrt. the Ideal NDCG, i.e. documents ranked in descending order of graded label.
    """
    __name__ = 'nDCG'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'cutoff': measures.ParamInfo(dtype=int, required=False, desc='ranking cutoff threshold'),
        'dcg': measures.ParamInfo(dtype=str, choices=['log2', 'exp-log2'], default='log2', desc='DCG formulation')
    }


nDCG = _nDCG()
NDCG = nDCG
measures.register(nDCG, ['NDCG'])



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _R(measures.BaseMeasure):
    """
    Recall@k (R@k). The fraction of relevant documents for a query that have been retrieved by rank k.
    """
    __name__ = 'R'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'cutoff': measures.ParamInfo(dtype=int, required=True, desc='ranking cutoff threshold'),
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


R = _R()
measures.register(R)



registry = {}
def register(measure, aliases=[], name=None):
    if name is None:
        name = measure.__name__
    assert name not in registry
    registry[name] = measure
    for alias in aliases:
        assert alias not in registry
        registry[alias] = measure
    return registry

from .base import BaseMeasure, ParamInfo, MultiMeasures, MeanAgg, SumAgg
from .ap import AP, MAP, _AP
from .bpref import Bpref, BPref, _Bpref
from .err import ERR, _ERR
from .infap import infAP, _infAP
from .iprec import IPrec, _IPrec
from .judged import Judged, _Judged
from .ndcg import nDCG, NDCG, _nDCG
from .numq import NumQ, _NumQ
from .numrel import NumRel, _NumRel
from .numret import NumRet, NumRelRet, _NumRet
from .p import P, _P
from .r import R, _R
from .rbp import RBP, _RBP
from .rprec import Rprec, _Rprec
from .rr import RR, _RR
from .setp import SetP, _SetP
from .success import Success, _Success



__version__ = "0.1.0"
from . import util
from .util import convert_trec_name
from . import measures
from .measures import P, RR, Rprec, AP, nDCG, R, Bpref, Judged, ERR, RBP, NumRet, NumRelRet, NumQ, NumRel, SetP, Success, IPrec, infAP
from . import providers


DefaultPipeline = providers.FallbackProvider([
	providers.PytrecEvalProvider(),
	# providers.TrectoolsProvider(),  # buggy; will add back later
	providers.JudgedProvider(),
	providers.GdevalProvider(),
	providers.MsMarcoProvider(),
])
calc_ctxt = DefaultPipeline.calc_ctxt
iter_calc = DefaultPipeline.iter_calc
calc_aggregate = DefaultPipeline.calc_aggregate



import unittest
import itertools
import ir_measures
from ir_measures import AP, P, nDCG, NumRel, NumRelRet, Bpref, NumQ, RR, Rprec, NumRet, IPrec

class TestConvertTrecName(unittest.TestCase):

    def test_convert_trec_name(self):
        cases = {
            'map': [AP],
            'P_5': [P@5],
            'P_5,10': [P@5, P@10],
            'P.5': [P@5],
            'P.5,10': [P@5, P@10],
            'P': [P@5, P@10, P@15, P@20, P@30, P@100, P@200, P@500, P@1000],
            'ndcg': [nDCG],
            'ndcg_cut.10': [nDCG@10],
            'ndcg_cut.5,10': [nDCG@5, nDCG@10],
            'ndcg_cut_10': [nDCG@10],
            'ndcg_cut_5,10': [nDCG@5, nDCG@10],
            'ndcg_cut': [nDCG@5, nDCG@10, nDCG@15, nDCG@20, nDCG@30, nDCG@100, nDCG@200, nDCG@500, nDCG@1000],
            'official': [P@5, P@10, P@15, P@20, P@30, P@100, P@200, P@500, P@1000, Rprec, Bpref, IPrec@0.0, IPrec@0.1, IPrec@0.2, IPrec@0.3, IPrec@0.4, IPrec@0.5, IPrec@0.6, IPrec@0.7, IPrec@0.8, IPrec@0.9, IPrec@1.0, AP, NumQ, NumRel, NumRelRet, NumRet, RR],
        }
        for case in cases:
            with self.subTest(case):
                self.assertEqual(ir_measures.convert_trec_name(case), cases[case])


if __name__ == '__main__':
    unittest.main()



import contextlib
from ir_measures import providers, measures
from ir_measures.util import QrelsConverter, RunConverter, flatten_measures


class FallbackProvider(providers.MeasureProvider):
    def __init__(self, providers):
        super().__init__()
        self.providers = providers

    @contextlib.contextmanager
    def _calc_ctxt(self, measures, qrels):
        measures = flatten_measures(measures)
        provider_measure_pairs = []
        for provider in self.providers:
            if not provider.is_available():
                continue
            provider_measures = set()
            for measure in measures:
                if provider.supports(measure):
                    provider_measures.add(measure)
            if provider_measures:
                measures = measures - provider_measures
                provider_measure_pairs.append((provider, provider_measures))
                if not measures:
                    break
        if measures:
            raise ValueError(f'unsupported measures {measures}')

        with contextlib.ExitStack() as stack:
            contexts = []
            qrels_teed = QrelsConverter(qrels).tee(len(provider_measure_pairs))
            for (provider, provider_measures), qrels in zip(provider_measure_pairs, qrels_teed):
                contexts.append(stack.enter_context(provider.calc_ctxt(provider_measures, qrels.qrels)))
            def _iter_calc(run):
                runs_teed = RunConverter(run).tee(len(contexts))
                for ctxt, run in zip(contexts, runs_teed):
                    yield from ctxt(run.run)
            yield _iter_calc

    def supports(self, measure):
        for provider in self.providers:
            if provider.is_available() and provider.supports(measure):
                return True
        return False



import contextlib
import ir_measures
from ir_measures import providers, measures
from ir_measures.providers.base import Any, Choices, Metric, NOT_PROVIDED
from ir_measures.bin import msmarco_eval
import sys

class MsMarcoProvider(providers.MeasureProvider):
    NAME = 'msmarco'
    SUPPORTED_MEASURES = [
        measures._RR(cutoff=Any(), rel=Any()),
    ]

    @contextlib.contextmanager
    def _calc_ctxt(self, measures, qrels):
        measures = ir_measures.util.flatten_measures(measures)
        qrels = ir_measures.util.QrelsConverter(qrels).as_dict_of_dict()

        qrels_by_rel = {}
        for measure in measures:
            rel = measure['rel']
            if rel not in qrels_by_rel:
                qrels_by_rel[rel] = {qid: {did: score for did, score in dids.items() if score >= rel} for qid, dids in qrels.items()}

        del qrels # reference no longer needed

        def _iter_calc(run):
            run = ir_measures.util.RunConverter(run).as_dict_of_dict()
            sorted_run = {q: list(sorted(run[q].items(), key=lambda x: (-x[1], x[0]))) for q in run}
            sorted_run = {q: [did for did, _ in v] for q, v in sorted_run.items()}
            for measure in ir_measures.util.flatten_measures(measures):
                rel = measure['rel']
                cutoff = measure['cutoff']
                if cutoff is NOT_PROVIDED:
                    cutoff = sys.maxsize
                msmarco_result = msmarco_eval.compute_metrics(qrels_by_rel[rel], sorted_run, max_rank=cutoff)
                for qid, value in msmarco_result[f'MRR @{cutoff} by query'].items():
                    yield Metric(query_id=qid, measure=measure, value=value)
        yield _iter_calc


providers.register(MsMarcoProvider())



import argparse
import ir_measures
from ir_measures.util import GenericScoredDoc, GenericQrel


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('qrels')
    parser.add_argument('run')
    args = parser.parse_args()
    run = (l.split() for l in open(args.run))
    run = (GenericScoredDoc(cols[0], cols[2], float(cols[4])) for cols in run)
    qrels = (l.split() for l in open(args.qrels))
    qrels = (GenericQrel(cols[0], cols[2], int(cols[3])) for cols in qrels)
    s, c = 0, 0
    for result in (ir_measures.Judged@5).iter_calc(qrels, run):
        s += result.value
        c += 1
    print(s/c)


if __name__ == '__main__':
    main()



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _RR(measures.BaseMeasure):
    """
    The [Mean] Reciprocal Rank ([M]RR) is a precision-focused measure that scores based on the reciprocal of the rank of the
    highest-scoring relevance document. An optional cutoff can be provided to limit the
    depth explored. rel (default 1) controls which relevance level is considered relevant.
    """
    __name__ = 'RR'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'cutoff': measures.ParamInfo(dtype=int, required=False, desc='ranking cutoff threshold'),
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


RR = _RR()
measures.register(RR)



import contextlib
from collections import namedtuple
from ir_measures import providers, measures


class MeasureProvider:
    NAME = None
    SUPPORTED_MEASURES = []

    def __init__(self):
        self._is_available = None

    def calc_ctxt(self, measures, qrels):
        if self.is_available():
            return self._calc_ctxt(measures, qrels)
        else:
            raise RuntimeError('provider not available')

    def iter_calc(self, measures, qrels, run):
        if self.is_available():
            return self._iter_calc(measures, qrels, run)
        else:
            raise RuntimeError('provider %s not available' % self.NAME)

    @contextlib.contextmanager
    def _calc_ctxt(self, measures, qrels):
        # Implementations must either provide _calc_ctxt or _iter_calc
        def _iter_calc(run):
            yield from self.iter_calc(measures, qrels, run)
        yield _iter_calc

    def _iter_calc(self, measures, qrels, run):
        # Implementations must either provide _calc_ctxt or _iter_calc
        with self.calc_ctxt(measures, qrels) as _iter_calc:
            yield from _iter_calc(run)

    def calc_aggregate(self, measures, qrels, run):
        aggregators = {m: m.aggregator() for m in measures}
        for metric in self.iter_calc(measures, qrels, run):
            aggregators[metric.measure].add(metric.value)
        return {m: agg.result() for m, agg in aggregators.items()}

    def supports(self, measure):
        measure.validate_params()
        for supported_measure in self.SUPPORTED_MEASURES:
            if supported_measure.NAME == measure.NAME:
                valid = True
                for param_name, param_spec in supported_measure.params.items():
                    if not param_spec.validate(measure[param_name]):
                        valid = False
                        break
                if valid:
                    return True
        return False

    def is_available(self):
        if self._is_available is not None:
            return self._is_available
        try:
            self.initialize()
            self._is_available = True
        except RuntimeError:
            self._is_available = False
        return self._is_available

    def initialize(self):
        pass



class ParamSpec:
    def validate(self, value):
        raise NotImplementedError()

class Any:
    def __init__(self, required=False):
        self.required = required

    def validate(self, value):
        if self.required:
            return value is not NOT_PROVIDED
        return True

class Choices:
    def __init__(self, *args):
        self.choices = args

    def validate(self, value):
        return value in self.choices

NOT_PROVIDED = measures.base._NOT_PROVIDED

Metric = namedtuple('Metric', ['query_id', 'measure', 'value'])



from ir_measures import measures
from .base import BaseMeasure, ParamInfo


class _SetP(measures.BaseMeasure):
    """
    The Set Precision (SetP); i.e., the number of relevant docs divided by the total number retrieved
    """
    __name__ = 'SetP'
    NAME = __name__
    SUPPORTED_PARAMS = {
        'rel': measures.ParamInfo(dtype=int, default=1, desc='minimum relevance score to be considered relevant (inclusive)')
    }


SetP = _SetP()
measures.register(SetP)
